#
# This is the script to generate the test cases.
# Run with python, not pypy (pypy produces different random numbers)
#
def write(fname, i, n, edges):
    print "cat > %s%03d.in << EOF" % (fname, i)
    print n, len(edges)
    for a, b in edges:
        print a + 1, b + 1
    print "EOF"

#
# a binary DAG connecting i to 2*i + 1/2
#
def makeheapliketree(n):
    edges = []
    c = 0
    while 2*c + 2 < n:
        edges.append((2*c + 1, c))
        edges.append((2*c + 2, c))
        c += 1

    return edges

from random import seed
seed(42)
from random import randint
from random import shuffle

#
# random DAG with n nodes and m edges, possibly disconnected
#
def makerandomdag(n, m):
    edges = set()
    while len(edges) < m:
        u = randint(0, n-2)
        v = randint(u+1, n-1)
        if (u, v) in edges:
            continue

        edges.add((u, v))

    return list(edges)

def makelongchain(n):
    return [(i, i+1) for i in range(n-1)]

def makelongchainshuffled(n):
    edges = [(i+1, i) for i in range(n-1)]
    shuffle(edges)
    return edges

def lotsofsmalldisconnected(n, l):
    """ graph consists of many 1 -> 2 -> ... -> l chains. """ 
    edges = []
    k = 0
    while len(edges) + l < 100000 and k + l - 1 < n:
        edges.extend([(j, j+1) for j in range(k, k + l - 1)])
        k += l

    return randomrenumber(n, edges)

def randomrenumber(n, edges):
    """
    The generators so far assume i < j to ensure the absence of cycles
    Randomly renumber a graph so that i < j doesn't hold.
    """
    perm = range(n)
    shuffle(perm)
    return [(perm[i], perm[j]) for i, j in edges]

for i, sz in enumerate([10, 50, 1000, 100000]):
    write("heaplike", i+1, sz, makeheapliketree(sz))

for i, (n, m) in enumerate([(10, 10), (10, 30), (50, 50), (50, 500), (1000, 5000), (50000, 100000), (100000, 100000)]):
    write("random", i+1, n, makerandomdag(n, m))

write("heaplike", 5, 100000, randomrenumber(100000, makeheapliketree(100000)))

# 3 more random ones, this time renumbered
for j in range(3):
    i += 1
    write("random", i+1, 100000, randomrenumber(100000, makerandomdag(100000, 100000)))

# just one train
write("sparse", 1, 2, [(0, 1)])
write("sparse", 2, 10, [(3, 7)])
write("sparse", 3, 100000, [(4, 100)])

# no trains at all
write("notrains", 1, 1, [])
write("notrains", 2, 10, [])
write("notrains", 3, 100000, [])

# a long linear chain
write("longchain", 1, 10, makelongchain(10))
write("longchain", 2, 100000, makelongchain(100000))

# same shuffled
write("longchainshuffled", 1, 10, makelongchainshuffled(10))
write("longchainshuffled", 2, 100000, makelongchainshuffled(100000))
write("longchainshuffled", 3, 100000, randomrenumber(100000, makelongchainshuffled(100000)))

write("lotsofsmall", 1, 10, lotsofsmalldisconnected(10, 2))
write("lotsofsmall", 2, 10, lotsofsmalldisconnected(10, 3))
write("lotsofsmall", 3, 100000, lotsofsmalldisconnected(100000, 2))
write("lotsofsmall", 4, 100000, lotsofsmalldisconnected(100000, 3))
write("lotsofsmall", 5, 100000, lotsofsmalldisconnected(100000, 10))

