#
# This is the script to generate the test cases.
# Run with python, not pypy (pypy produces different random numbers)
#
from collections import defaultdict

count = defaultdict(int)
def write(fname,  n, edges):
    count[fname] += 1
    print "cat > %s%03d.in << EOF" % (fname, count[fname])
    print n, len(edges)
    for a, b in edges:
        print a + 1, b + 1
    print "EOF"

from random import seed
seed(42)
from random import randint
from random import shuffle

def randomrenumber(n, edges):
    """
    The generators so far assume i < j to ensure the absence of cycles
    Randomly renumber a graph so that i < j doesn't hold.
    """
    perm = range(n)
    shuffle(perm)
    return [(perm[i], perm[j]) for i, j in edges]

def makedifficult(n):
    """
    Try to make a difficult case
    1  2  3  4   5 ....
     \/ \ /\ /\ / \
     6   7  8  9  10

    """
    edges1 = [(i, i+n/2) for i in range(n/2)]
    edges2 = [(i+1, i+n/2) for i in range(n/2-1)]
    return edges1 + edges2


#
# test case generation starts, don't change the order if you want to preserve already generated 
# cases.
#
write("comb", 10, makedifficult(10))
write("comb", 10, randomrenumber(10, makedifficult(10)))
write("comb", 100000, makedifficult(100000))
write("comb", 100000, randomrenumber(100000, makedifficult(100000)))
write("comb", 100000, randomrenumber(100000, makedifficult(100000)))

